from enum import Enum
import grpc
from json import dumps, loads
import os
from robomotion.utils import File
from robomotion import plugin_pb2
from robomotion import plugin_pb2_grpc


class AttachConfig:
    def __init__(self, protocol: str, addr: str, pid: int, namespace: str):
        self.protocol = protocol
        self.addr = addr
        self.pid = pid
        self.namespace = namespace


class EProtocol(Enum):
    ProtocolInvalid = ''
    ProtocolNetRPC = 'netrpc'
    ProtocolGRPC = 'grpc'


class Debug:
    @staticmethod
    def attach(g_addr: str, ns: str):
        cfg = AttachConfig(str(EProtocol.ProtocolGRPC.value), g_addr, os.getpid(), ns)
        cfg_data = dumps(cfg.__dict__)
        addr = Debug.get_rpc_addr()
        if addr == '':
            print('runner RPC address is None')
            exit(0)

        channel = grpc.insecure_channel(addr)
        client = plugin_pb2_grpc.DebugStub(channel)

        request = plugin_pb2.AttachRequest(config=cfg_data.encode())
        client.Attach(request)
        channel.close()

    @staticmethod
    def detach(ns: str):
        addr = Debug.get_rpc_addr()
        if addr == '':
            print('runner RPC address is None')
            exit(0)

        channel = grpc.insecure_channel(addr)
        client = plugin_pb2_grpc.DebugStub(channel)

        request = plugin_pb2.DetachRequest(namespace=ns)
        client.Detach(request)
        channel.close()

    @staticmethod
    def get_rpc_addr() -> str:
        dir = File.temp_dir()
        path = os.path.join(dir, 'runcfg.json')

        f = open(path, 'r')
        data = f.read()
        obj = loads(data)
        return str(obj['listen']).replace('[::]', '127.0.0.1')



from datetime import datetime as dt
from setuptools import find_packages, setup

PROJECT_NAME = "robomotion"
PROJECT_PACKAGE_NAME = "robomotion"
PROJECT_LICENSE = "Apache License 2.0"
PROJECT_AUTHOR = "Robomotion IO"
PROJECT_COPYRIGHT = f" 2020-{dt.now().year}, {PROJECT_AUTHOR}"
PROJECT_URL = "https://robomotion.io/"
PROJECT_EMAIL = "support@robomotion.io"

PROJECT_GITHUB_USERNAME = "robomotionio"
PROJECT_GITHUB_REPOSITORY = "python-lib"

PYPI_URL = f"https://pypi.org/project/{PROJECT_PACKAGE_NAME}"
GITHUB_PATH = f"{PROJECT_GITHUB_USERNAME}/{PROJECT_GITHUB_REPOSITORY}"
GITHUB_URL = f"https://github.com/{GITHUB_PATH}"

PROJECT_URLS = {
    "Bug Reports": f"{GITHUB_URL}/issues",
    "Dev Docs": "https://docs.robomotion.io/",
    "Slack": "https://slack.robomotion.io",
    "Forum": "https://forum.robomotion.io/",
}

PACKAGES = find_packages(exclude=["tests", "tests.*"])

REQUIRES = [
    "protobuf==3.8.0",
    "grpcio==1.36.1",
    "grpcio-tools==1.36.1",
    "zipp==3.1.0"
]

setup(
    name=PROJECT_PACKAGE_NAME,
    version="1.4.1",
    url=PROJECT_URL,
    project_urls=PROJECT_URLS,
    author=PROJECT_AUTHOR,
    author_email=PROJECT_EMAIL,
    packages=PACKAGES,
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIRES,
    python_requires='>=3.6',
    test_suite="tests",
)